/*
 * Copyright (c) 2005 Sendmail, Inc. and its suppliers.
 *	All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 */

#include "sm/generic.h"
SM_IDSTR(id, "@(#)$Id: t-batv-cnf-0.c,v 1.3 2005/02/28 23:46:16 ca Exp $")

#include "sm/error.h"
#include "sm/sm-conf.h"
#include "sm/sm-conf-prt.h"
#include "sm/batv.h"
#include "sm/batvcnf.h"
#define SM_BATVCNFDEF	1
#include "sm/batvcnfdef.h"
#include "sm/io.h"
#include "sm/test.h"

#if SM_USE_BATV
static int Verbose;

static sm_ret_T
prt_batv_keys(batv_key_T batv_key)
{
	sm_io_fprintf(smioerr
		, "key=%d, private='%s'\n"
		, batv_key.batv_key_number
		, batv_key.batv_priv_key
		);
	return 0;
}

static sm_ret_T
prt_batv_cnf(batv_cnf_P batv_cnf)
{
	uint i;

	for (i = 0; i < batv_cnf->batvcnf_n; i++)
		prt_batv_keys(batv_cnf->batvcnf_keys[i]);
	return 0;
}


static sm_ret_T
test(const char *path)
{
	int err;
	batv_cnf_T batv_cnf;
	FILE *fp;
	sm_conf_T *smc;

	fp = NULL;
	sm_memzero(&batv_cnf, sizeof(batv_cnf));
	smc = sm_conf_new(path);
	if (smc == NULL)
	{
		err = errno;

		sm_io_fprintf(smioerr,
			"sev=ERROR, func=sm_readconf, file=%s, sm_conf_new=NULL, errno=%d\n",
			path, err);
		return ENOMEM;
	}
	err = sm_conf_read_FILE(smc, path, fp);
	if (err != 0)
	{
		sm_prt_conferr(path, smc, err);
		sm_conf_destroy(smc);
		smc = NULL;
		return err;
	}

	err = sm_conf_scan(smc, batv_defs, 0, &batv_cnf);
	if (err != 0)
	{
		sm_prt_conferr(path, smc, err);
	}
	else
		prt_batv_cnf(&batv_cnf);
	return 0;
}

int
main(int argc, char *argv[])
{
	int c, r;
	char *prg;

	opterr = 0;
	Verbose = 0;
	prg = argv[0];
	while ((c = getopt(argc, argv, "V")) != -1)
	{
		switch (c)
		{
		  case 'V':
			++Verbose;
			break;
		  default:
			exit(1);
		}
	}
	sm_test_begin(argc, argv, "test batv conf 0");
	argc -= optind;
	argv += optind;
	if (argc > 0)
	{
		for (r = 0; r < argc; r++)
		{
			test(argv[r]);
		}
	}
	return sm_test_end();
}
#else /* SM_USE_BATV */
int
main(int argc, char *argv[])
{
	sm_io_fprintf(smioerr, "SM_USE_BATV not available\n");
	return 0;
}
#endif /* SM_USE_BATV */
